#if !defined(IOSTREAM)
        #define IOSTREAM
        #include <iostream.h>
#endif

#if !defined(VECTOR3D)
        #define VECTOR3D
        #include "vector3d.h"
#endif

#if !defined(MATH)
        #define MATH
        #include <math.h>
#endif

#if !defined(MYMATH)
        #define MYMATH
        #include "MyMath.h"
#endif


// this class is designed to handle a unit quaternion and not a general quaternion
// the quaternion is handled using a generalised version of the spherical coordinates
// ie theta, alpha, beta. alpha and beta are the usual spherical coordinates to
// describe a unit vector U. theta is the last coordinates so that the quaternion
// represents a rotation of 2*theta around U
// we have theta E [0,pi], alpha E [0,pi], beta E [0, 2pi]


class quaternion
{
public:
	// ---------------------------------------------------------------------
	// ---------             constructor and destructor          -----------

	quaternion(void);
		//  default constructor, rotation of 0 around e1

	quaternion(vector3d v, double t=kpi/2.0);
		// rotation of 2t around v

	quaternion(double t, double a, double b);
		// handly supplied quaternion
	
	// ---------------------------------------------------------------------
	// ---------                 data access                 ---------------

	double& operator[](const int i);
		// return the coordinate as 
		// theta <-> 1
		// alpha <-> 2
		// beta  <-> 3	

	vector3d vect(void);
		// return the vector in cartesian coordinates
	
	double RotationAngle(void){return 2.0*theta;}
		// return the rotation angle

	friend ostream& operator<<(ostream& out, quaternion& q);
 		// send q to the stream out       

	ostream& display(ostream& out=cout);
		// send q to the stream out using cartesian representation

	// ---------------------------------------------------------------------
	// ---------------                   services           ----------------

	quaternion operator=(const quaternion& q);
		// affectation operator
	
	quaternion operator~(void);
		// return the conjugate quaternion

	friend quaternion operator*(const quaternion& a, const quaternion& b);
		// return a*b, be carefull of the order
	
	friend istream& operator>>(istream& in, quaternion &q);
		// read the quaternion from in using the format form operator<<

	void move(double dt, double da=0.0, double db=0.0);
		// move theta, alpha and beta respectively of dt, da and db
		// taking care of the modulo

	vector3d rotate(vector3d u);
		// return the image of u by the rotation represented by the quaternion

private:
	double theta, alpha, beta;
	double dummy;	// dummy variable to store dummy data
};
